from django.contrib import admin

# Register your models here.
# vim: set fileencoding=utf-8 :
from django.contrib import admin

import django_moncash.models as models


class TransactionAdmin(admin.ModelAdmin):

    list_display = (
        'id',
        'user',
        'order_id',
        'amount',
        'status',
        'return_url',
        'meta_data',
        'created_at',
        'updated_at',
    )
    list_filter = (
        'user',
        'created_at',
        'updated_at',
        'amount',
        'status',
        'return_url',
    )
    date_hierarchy = 'created_at'


def _register(model, admin_class):
    admin.site.register(model, admin_class)


_register(models.Transaction, TransactionAdmin)



from django.conf import settings 


from .models import Transaction


import moncash

from moncash.exceptions import  NotFoundError

environment = moncash.environment.Sandbox

if str.lower(settings.MONCASH['ENVIRONMENT']) == 'production':
    environment = moncash.environment.Production

gateway = moncash.Moncash(
    client_id=settings.MONCASH['CLIENT_ID'],
    client_secret=settings.MONCASH['SECRET_KEY'],
    environment= environment
)

def init_payment(request,amount: float,return_url: str = None,order_id: str=None,meta_data: dict=None):

    amount = float(amount)

    if not return_url:
        return_url = request.get_full_path()

    if order_id:
        transaction = Transaction.objects.create(user=request.user,amount=amount,return_url=return_url,meta_data=meta_data,order_id=order_id)
    else:
        transaction = Transaction.objects.create(user=request.user,amount=amount,return_url=return_url,meta_data=meta_data)

    payment_url = gateway.payment.create(amount=transaction.amount, reference=str(transaction.order_id))

    return {
        "payment_url":payment_url,
        "transaction":transaction
    }


def verify_payment(request,moncash_transaction_id: str=None):

    if not moncash_transaction_id:
        moncash_transaction_id = request.GET.get("transactionId",None)

    transaction = {}

    try:
        transaction = gateway.payment.get_by_id(transactionId=moncash_transaction_id)
    except NotFoundError:
        transaction["reference"] = None


    try:
        transaction = Transaction.objects.get(order_id=transaction["reference"])
    except Transaction.DoesNotExist:
        transaction = None

    return {
        "transaction": transaction,
        "transactionId":moncash_transaction_id
    }


def consume_payment(request,moncash_transaction_id: str=None):

    payment = None

    if moncash_transaction_id:
        payment = verify_payment(moncash_transaction_id=moncash_transaction_id)
    else:
        payment = verify_payment(request)

    if payment['transaction']:

        if payment['transaction'].status == Transaction.Status.CONSUME:
            return {
                "success":False,
                "error":"USED",
                "payment":payment
            }
        
        payment['transaction'].status = Transaction.Status.CONSUME
        payment['transaction'].save()

        return {
            "success":True,
            "payment":payment
        }
            
      
    return {
        "success":False,
        "error":"NOT_FOUND"
    }


from django.shortcuts import redirect,resolve_url

from django.http import Http404

from .utils import verify_payment
from .models import Transaction

from ._utils import add_params

def complete_transaction(request):

    payment = verify_payment(request)

    transaction:Transaction = payment['transaction']

    if transaction:
        transaction.status = Transaction.Status.COMPLETE
        transaction.save()

        return redirect(add_params(resolve_url(transaction.return_url),{"transactionId":payment['transactionId']}))
    else:
        raise Http404


from django.db import models

from django.contrib.auth import get_user_model 

import uuid

from django.utils.translation import gettext_lazy as _

User = get_user_model()

class Transaction(models.Model):

    class Status(models.TextChoices):
        PENDING = 'PENDING', _("Pending")
        COMPLETE = 'COMPLETE', _("Complete")
        CONSUME = 'CONSUME', _("Consume")


    user = models.ForeignKey(
        User,
        on_delete=models.SET_NULL,
        null=True,
        blank=True
    )

    order_id = models.CharField(_("Order id"), max_length=50,default=uuid.uuid4,unique=True, editable=False)
    amount   = models.DecimalField(_("Amount"), max_digits=11, decimal_places=2,blank=False,null=False)
    status = models.CharField(_('Status'),max_length=25,choices=Status.choices,default=Status.PENDING,blank=False)

    return_url = models.TextField(_("Return URL"),blank=False,null=False)

    meta_data = models.JSONField(_("Meta data"),null=True,blank=True)
    
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)



import urllib.parse as urlparse
from urllib.parse import urlencode

def add_params(url,params):

    url_parts = list(urlparse.urlparse(url))
    query = dict(urlparse.parse_qsl(url_parts[4]))
    query.update(params)

    url_parts[4] = urlencode(query)

    return urlparse.urlunparse(url_parts)


# Generated by Django 4.2 on 2023-04-07 15:39

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order_id', models.CharField(default=uuid.uuid4, editable=False, max_length=50, unique=True, verbose_name='Order id')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=11, verbose_name='Amount')),
                ('status', models.CharField(choices=[('PENDING', 'Pending'), ('COMPLETE', 'Complete'), ('CONSUME', 'Consume')], default='PENDING', max_length=25, verbose_name='Status')),
                ('return_url', models.TextField(verbose_name='Return URL')),
                ('meta_data', models.JSONField(blank=True, null=True, verbose_name='Meta data')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
